using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Petzold.Xna3D
{
    public class Cone : Primitive
    {
        // Cone: Sits on XZ axis, point upwards
        public Cone(GraphicsDevice graphicsDevice, float radius, float length, int slices, Color clr, Matrix matrix) :
            base(graphicsDevice)
        {
            // Vertices
            VertexPositionNormalColor[] vertices = new VertexPositionNormalColor[2 * slices];
            float normalOffset = radius * radius / length;

            for (int slice = 0; slice < slices; slice++)
            {
                float angle = slice * MathHelper.TwoPi / slices;
                float x = radius * (float)Math.Sin(angle);
                float z = radius * (float)Math.Cos(angle);

                Vector3 normal = new Vector3(x, normalOffset, z);
                normal.Normalize();
                vertices[2 * slice + 0] = new VertexPositionNormalColor(new Vector3(x, 0, z), normal, clr);
                vertices[2 * slice + 1] = new VertexPositionNormalColor(new Vector3(0, length, 0), normal, clr);
            }

            // Indices
            short[] indices = new short[3 * slices];

            for (int slice = 0; slice < slices; slice++)
            {
                indices[3 * slice + 0] = (short)(2 * slice + 0);
                indices[3 * slice + 1] = (short)(2 * slice + 1);
                indices[3 * slice + 2] = (short)((2 * slice + 2) % (2 * slices));
            }

            Initialize(vertices, indices, matrix);
        }
    }
}
