using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Petzold.Xna3D
{
    public class Cylinder : Primitive
    {
        // Cylinder: Extends up positive Y axis
        public Cylinder(GraphicsDevice graphicsDevice, float radius, float length, int slices, Color clr, Matrix matrix) :
            base(graphicsDevice)
        {
            this.graphicsDevice = graphicsDevice;

            // Vertices
            VertexPositionNormalColor[] vertices = new VertexPositionNormalColor[2 * slices];
            vertices[0] = new VertexPositionNormalColor(Vector3.Zero, Vector3.Down, clr);

            for (int slice = 0; slice < slices; slice++)
            {
                float angle = slice * MathHelper.TwoPi / slices;
                float x = radius * (float)Math.Sin(angle);
                float z = radius * (float)Math.Cos(angle);

                vertices[2 * slice + 0] = new VertexPositionNormalColor(new Vector3(x, 0, z), new Vector3(x, 0, z), clr);
                vertices[2 * slice + 1] = new VertexPositionNormalColor(new Vector3(x, length, z), new Vector3(x, 0, z), clr);
            }

            // Indices
            short[] indices = new short[6 * slices];

            for (int slice = 0; slice < slices; slice++)
            {
                indices[6 * slice + 0] = (short)(2 * slice + 0);
                indices[6 * slice + 1] = (short)(2 * slice + 1);
                indices[6 * slice + 2] = (short)((2 * slice + 2) % (2 * slices));
                indices[6 * slice + 3] = (short)(2 * slice + 1);
                indices[6 * slice + 4] = (short)((2 * slice + 3) % (2 * slices));
                indices[6 * slice + 5] = (short)((2 * slice + 2) % (2 * slices)); 
            }

            Initialize(vertices, indices, matrix);
        }
    }
}
