using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Petzold.Xna3D
{
    public class Disk : Primitive
    {
        // Disk: Sits on on XZ plane, facing down
        public Disk(GraphicsDevice graphicsDevice, float radius, int slices, Color clr, Matrix matrix) :
            base(graphicsDevice)
        {
            this.graphicsDevice = graphicsDevice;

            // Vertices
            VertexPositionNormalColor[] vertices = new VertexPositionNormalColor[slices + 1];
            vertices[0] = new VertexPositionNormalColor(Vector3.Zero, Vector3.Down, clr);

            for (int slice = 0; slice < slices; slice++)
            {
                float angle = slice * MathHelper.TwoPi / slices;
                float x = radius * (float)Math.Sin(angle);
                float z = radius * (float)Math.Cos(angle);

                vertices[slice + 1] = new VertexPositionNormalColor(new Vector3(x, 0, z), Vector3.Down, clr);
            }

            // Indices
            short[] indices = new short[3 * slices];

            for (int slice = 0; slice < slices; slice++)
            {
                indices[3 * slice + 0] = (short)(0);
                indices[3 * slice + 1] = (short)(1 + slice);
                indices[3 * slice + 2] = (short)(1 + (slice + 1) % slices); 
            }

            Initialize(vertices, indices, matrix);
        }
    }
}
