using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Petzold.Xna3D
{
    public abstract class Primitive
    {
        protected GraphicsDevice graphicsDevice;
        protected VertexBuffer vertexBuffer;
        protected IndexBuffer vertexIndices;

        protected Primitive(GraphicsDevice graphicsDevice)
        {
            this.graphicsDevice = graphicsDevice;
        }

        protected void Initialize(VertexPositionNormalColor[] vertices, short[] indices, Matrix matrix)
        {
            // Transform the vertices and normals
            if (matrix != Matrix.Identity)
            {
                for (int i = 0; i < vertices.Length; i++)
                    vertices[i].Position = Vector3.Transform(vertices[i].Position, matrix);

                matrix.M41 = matrix.M42 = matrix.M43 = 0;

                for (int i = 0; i < vertices.Length; i++)
                    vertices[i].Normal = Vector3.Transform(vertices[i].Normal, matrix);
            }

            // Create the VertexBuffer 
            vertexBuffer = new VertexBuffer(graphicsDevice, typeof(VertexPositionNormalColor),
                                            vertices.Length, BufferUsage.WriteOnly);

            vertexBuffer.SetData<VertexPositionNormalColor>(vertices);

            // Create the IndexBuffer
            vertexIndices = new IndexBuffer(graphicsDevice, IndexElementSize.SixteenBits,
                                            indices.Length, BufferUsage.WriteOnly);

            vertexIndices.SetData<short>(indices);
        }

        public void Draw()
        {
            graphicsDevice.SetVertexBuffer(vertexBuffer);
            graphicsDevice.Indices = vertexIndices;

            graphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList,
                                                 0, 0, vertexBuffer.VertexCount,
                                                 0, vertexIndices.IndexCount / 3);
        }
    }
}
